/**
 * 
 */
package inra.ijpb.plugins;

import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ColorProcessor;
import ij.process.ImageProcessor;
import inra.ijpb.binary.BinaryImages;

/**
 * Select binary particles in a 2D or 3D image based on number of pixels.
 * 
 * @see AreaOpeningPlugin
 * 
 * @author David Legland
 *
 */
public class SizeOpeningPlugin implements PlugIn
{
	@Override
	public void run(String arg0)
	{
		ImagePlus imagePlus = IJ.getImage();
        
        // create the dialog, with operator options
		boolean isPlanar = imagePlus.getStackSize() == 1; 
		String title = isPlanar ? "Area Opening" : "Volume Opening";
        GenericDialog gd = new GenericDialog(title);
        String label = isPlanar ? "Min Pixel Number:" : "Min Voxel Number:";
        gd.addNumericField(label, 100, 0);
        gd.showDialog();
        
        // If cancel was clicked, do nothing
        if (gd.wasCanceled())
            return;

        int nPixelMin = (int) gd.getNextNumber();
        
        ImagePlus resultPlus;
        String newName = imagePlus.getShortTitle() + "-sizeOpen";
        
        if (isPlanar) 
        {
            ImageProcessor image = imagePlus.getProcessor();
            ImageProcessor result = null;
    		try 
    		{
    			result = BinaryImages.areaOpening(image, nPixelMin);
    		}
    		catch(RuntimeException ex)
    		{
    			IJ.error("Too many particles", ex.getMessage());
    			return;
    		}

            if (!(result instanceof ColorProcessor))
    			result.setLut(image.getLut());
            resultPlus = new ImagePlus(newName, result);    		
        }
        else
        {
            ImageStack image = imagePlus.getStack();
            ImageStack result = null;
    		try 
    		{
    			result = BinaryImages.volumeOpening(image, nPixelMin);
    		}
    		catch(RuntimeException ex)
    		{
    			IJ.error("Too many particles", ex.getMessage());
    			return;
    		}
        	result.setColorModel(image.getColorModel());
            resultPlus = new ImagePlus(newName, result);
        }
        
		resultPlus.copyScale(imagePlus);
        resultPlus.show();
		
		if (imagePlus.getStackSize() > 1) {
			resultPlus.setSlice(imagePlus.getSlice());
		}
	}
	

}
